# This file is a part of the LinkAhead Project.
#
# Copyright (C) 2018 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
# Copyright (C) 2019, 2020, 2025 Daniel Hornung <d.hornung@indiscale.com>
# Copyright (C) 2020 Henrik tom Wörden <h.tomwoerden@indiscale.com>
# Copyright (C) 2020, 2025 IndiScale <info@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#

###############################################################################
#                          header for patch scripts                           #
###############################################################################
#
# This file defines the following functions (see below for more detailed descriptions):
#
# - check_version_or_exit: Check if the stored version is as expected.
# - update_version: Set the stored version.
# - dump_table: Store table content into a file.
# - redo_table: Apply the table content stored by dump_table.
#
# This file also sets the `errexit` option (`set -e`).
#

set -e

# load useful functions #######################################################
source $UTILSPATH/load_settings.sh
source $UTILSPATH/helpers.sh

USAGE="$1 [ --env=ENV_FILE ] [ --patch=PATCH ] [ --backupdir=BACKUPDIR ]\n\n"
CMD_OPTIONS=$(cat <<EOF
options:

-h, --help
    Show brief help.
--env=ENV_FILE
    A file where variables are stored in the format of shell environment
    variables.  Content of this file overrides environment variables already
    present.
--patch=PATCH
    *TODO* Please document this option.
--backupdir=BACKUPDIR
    The directory where backup files are stored to.
--yaml[=True|False]
    If given (and the optional value is not False), additional machine-readable
    YAML output is emitted.  Only implemented for some commands yet.

This script basically uses the same environment variables as the server
configuration make file.  Notable examples are:

- LOGIN_PATH
- MYSQL_HOST
- MYSQL_PORT
- DATABASE_NAME
- DATABASE_USER
- DATABASE_USER_PW

EOF
           )

function _print_help() {
  echo -e "$USAGE"
  echo -e "$CMD_OPTIONS"
  if [ -n "$PRINT_HELP" ]; then
    echo -e "$PRINT_HELP"
  fi
  if [ -n "$1" ]; then
    echo -e "$1"
  fi
}

while test $# -gt 0; do
 case "$1" in
 -h|--help)
  _print_help
  exit 0
  ;;
 --env*)
  ENV_FILE="${1#--*=}"
  shift
  ;;
 --patch*)
  PATCH="${1#--*=}"
  PATCH_DIR=$(dirname $PATCH)
  shift
  ;;
 --backupdir*)
  BACKUPDIR="${1#--*=}"
  shift
  ;;
 --yaml*)
     if [[ "$1" == "--yaml" ]] ; then
         YAML="True"
     else
         YAML="${1#--*=}"
     fi
  shift
  ;;
 *)
  echo "Unknown option $1"
  exit 1
  shift
  ;;
 esac
done


if [ -n "$PATCH" ]; then
    echo -ne "applying patch $PATCH to $DATABASE_NAME ... "
fi

# usage: check_version_or_exit version
#
# Check if the version string is as expected.  Otherwise, `exit 0`.
#
# Arguments
# ---------
#
# version: str
#   The expected database version string, e.g. 'v2.0.0'
#
# Returns
# -------
#
# 0 on success
function check_version_or_exit {
    local version=$($MYSQL_CMD $(get_db_args) -B -e "Select CaosDBVersion();")
    local version="$(echo $version | sed 's/^CaosDBVersion()\s//')"
	  if [[ "$version" = "$1" ]]; then
        return 0
	  fi
    uptodate
}

# Deprecated name, call "check_version_or_exit" directly instead.
function check_version {
    check_version_or_exit "$1"
}

# usage: update_version version
#
# Set the version string.
#
# Arguments
# ---------
#
# version: str
#   The new database version string, e.g. 'v2.0.0'
function update_version {
    mysql_execute "DROP FUNCTION IF EXISTS CaosDBVersion; CREATE FUNCTION CaosDBVersion() RETURNS VARCHAR(255) DETERMINISTIC RETURN '$1';"
}

# usage: dump_table table
#
# Dump the table given in the argument into file "caosdb.<table>.<OLD_VERSION>.dump.sql"
function dump_table {
    if [[ -z $MYSQLDUMP_CMD ]]; then
        echo "Cannot find mysqldump program!" >&2
        exit 2
    fi
    $MYSQLDUMP_CMD $(get_db_args_nodb) $DATABASE_NAME $1 \
                    > ${DATABASE_NAME}.${1}.${OLD_VERSION}.dump.sql
}

# usage: redo_table table
#
# Apply the dump from `dump_table` to the database.  Takes the same argument.
function redo_table {
         $MYSQL_CMD $(get_db_args) < ${DATABASE_NAME}.${1}.${OLD_VERSION}.dump.sql
}


